#include "gsl/gsl_rng.h"
#include "gsl/gsl_randist.h"
/*#include "mex.h"*/

void range_sol(double *drec_mt, double *betao_mt, int nidx, double tol, double range[]);
void range_sol2(double *X, double *beta, double *dvec, double *a, int *ineq_flag, int dimX, int *idx, int nidx, double tol, double range[]);

int multinorm(double *omegap, int N, gsl_rng *r){
	
	int i1, i2;
	double *dtmpv1;
	double dtmp1;
	
	dtmpv1=(double *) mxMalloc(sizeof(double) * (N+1));
	
	cumsum(omegap,dtmpv1,N);
	
	dtmp1 = gsl_rng_uniform(r)*dtmpv1[N];
	i2=0;
	for(i1=0;i1<N;i1++){
		if(dtmp1>dtmpv1[i1]&&dtmp1<dtmpv1[i1+1]){
			i2=i1;
			/*mexPrintf("%d\n",i2);*/
			break;
		}/*else{
		break;
		}*/
	}
	/*mexPrintf("%d, %f, %f, %f, %f\n",i2,dtmp1,dtmpv1[1],dtmpv1[2],dtmpv1[N]);*/
	mxFree(dtmpv1);
	
	return i2;
}


void rmn_centered_gsl(double *CSig, double *tr, double *logp0, int Np, gsl_rng *r){
	
	double *tr0;
	int i1;
	
	
	tr0 = (double *) mxMalloc(sizeof(double) * Np);
	
	for(i1=0;i1<Np;i1++){
		tr0[i1] = gsl_ran_gaussian(r, 1.0);
	}
	
	MA_mul_vb(CSig, tr0, tr, Np, Np);
	
	/*
	for(i1=0;i1<Np;i1++){
	tr[i1] = tr0[i1] + mu[i1];
	}
	*/
	mxFree(tr0);
	
}


void Cal_MCmean_cov_colwise(double *Epara, int NMC, int Np, double *Empara, double *Covpara){
	
	int i1, i2, i3;
	int j1;
	
	/*double dtmp;
	
	//Empara = (double *) mxMalloc(sizeof(double) * Np);
	//Covpara = (double *) mxMalloc(sizeof(double) * Np * Np);	
	*/
	
	for(i2 = 0; i2<Np; i2++){
		Empara[i2] =0;
		for(i3 = 0; i3<Np;i3++){
			Covpara[i2 + i3*Np] = 0; 
		}
	}	
	
	for(i1 = 0; i1<NMC; i1++){		
		j1 = i1 * Np;
		
		for(i2 = 0; i2<Np; i2++){
			Empara[i2] = Empara[i2] + Epara[j1 + i2];
			/*mexPrintf("%f\n",Epara[j1 + i2]);*/
			for(i3 = 0; i3<Np;i3++){
				Covpara[i2 + i3*Np] = Covpara[i2 + i3*Np] + Epara[j1 + i2] * Epara[j1 + i3]; 
			}
		}		
	}
	
	for(i2 = 0; i2<Np; i2++){
		Empara[i2] /= NMC;	
		/*mexPrintf("%f\n",Empara[i2]);*/
	}
	
	for(i2 = 0; i2<Np; i2++){
		for(i3 = 0; i3<Np;i3++){
			Covpara[i2 + i3*Np] /= NMC;
			Covpara[i2 + i3*Np] -= Empara[i2]*Empara[i3];
			/*mexPrintf("%f\n",Covpara[i2 + i3*Np]);*/
		}
		/*mexPrintf("\n");*/
	}
	
}


void rmt(double *mu, double *CSig, double *tr, double *logp0, int Np, int df, long *idum){
	
	double *zr, *gr, *tr0;
	int i1, i2;
	double sdfd, dtmp=0;
	
	zr = (double *) mxMalloc(sizeof(double) * Np);
	gr = (double *) mxMalloc(sizeof(double) * Np);
	tr0 = (double *) mxMalloc(sizeof(double) * Np);
	
	sdfd = sqrt((double) df);
	
	for(i1=0;i1<Np;i1++){
		zr[i1] = gasdev(idum);
		gr[i1] = gamdev(df,idum);
		tr0[i1] = zr[i1]/sqrt(gr[i1]);
		tr[i1] = tr0[i1]*sdfd;
	}
	
	
	for(i1=0;i1<Np;i1++){
		dtmp += tr0[i1] * tr0[i1];
	}
	
	*logp0 = -0.5*((double) (Np+df))*log(1+dtmp); 
	
	MA_mul_vb(CSig, tr, tr0, Np, Np);
	for(i1=0;i1<Np;i1++){
		tr[i1] = tr0[i1] + mu[i1];
	}
	
	
	
	mxFree(tr0);
	mxFree(gr);
	mxFree(zr);
	
}

void rmt_gsl(double *mu, double *CSig, double *tr, double *logp0, int Np, int df, gsl_rng *r){
	
	double *zr, *gr, *tr0;
	int i1, i2;
	double sdfd, dtmp=0;
	double ddf = (double) df;
	
	zr = (double *) mxMalloc(sizeof(double) * Np);
	gr = (double *) mxMalloc(sizeof(double) * Np);
	tr0 = (double *) mxMalloc(sizeof(double) * Np);
	
	sdfd = sqrt((double) df);
	
	for(i1=0;i1<Np;i1++){
		zr[i1] = gsl_ran_gaussian(r, 1.0);
		gr[i1] = gsl_ran_gamma(r, ddf/2, 2.0) ;
		/*mexPrintf("%f,%f\n",zr[i1],gr[i1]);*/
		tr0[i1] = zr[i1]/sqrt(gr[i1]);
		tr[i1] = tr0[i1]*sdfd;
	}
	
	
	for(i1=0;i1<Np;i1++){
		dtmp += tr0[i1] * tr0[i1];
	}
	
	*logp0 = -0.5*((double) (Np+df))*log(1+dtmp); 
	
	MA_mul_vb(CSig, tr, tr0, Np, Np);
	for(i1=0;i1<Np;i1++){
		tr[i1] = tr0[i1] + mu[i1];
	}
	
	
	
	mxFree(tr0);
	mxFree(gr);
	mxFree(zr);
	
}

void dmt(double *mu, double *CSig, double *tr, double *logp0, int Np, int df){
	
	double *zr, *tr0;
	int i1, i2;
	double dtmp=0;
	double dNp = (double) Np;
	double ddf = (double) df;
	
	zr = (double *) mxMalloc(sizeof(double) * Np);
	tr0 = (double *) mxMalloc(sizeof(double) * Np);
	
	
	for(i1=0;i1<Np;i1++){
		tr0[i1] = tr[i1] - mu[i1];
	}
	
	forward_sol_c(CSig, zr, tr0, &Np, &Np);
	
	for(i1=0;i1<Np;i1++){
		dtmp += zr[i1] * zr[i1];
	}
	
	*logp0 = -0.5*((dNp + ddf))*log(1+dtmp/ddf); 	
	
	mxFree(tr0);
	mxFree(zr);
	
}

double dyn_likelihood(double *z, double *x, double *xv, double *dt,  double *para, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat)
/*
z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
para- a vector of proposed regression coefficients
length of NX+1, including a intercept at location NX.
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1=0, i2, i3, i4;
	int j1, j2, j3, j4;
	int k1, k2;
	int n1, n2;
	
	double *z1;
	double sum1 = 0;
	double dtmp1, dtmp2, dtmp;
	/*double p1, p2, p3;*/
	
	/* ***
	int N1m1=N0*(NR+1);
	int N1=N1m1+1;
	*/
	int N1m1;	
	int N1tmp;
	
	N1m1 = N1 - 1;
	
	for(i1=0;i1<NObs;i1++){
		/*i3=i1*N1;*/
		i4=i1*N1m1;
		i2=i1*N1*NLat;
		
		n1=i4*NXV;
		
		/* ***
		N1tmp=N0_ALL[i1]*(NR+1);
		*/
		/*N1tmp = obsloc[i1][N0_ALL[i1]];*/
		
		j3=i1*NX;
		
		/*p1=para[0]+para[8]*Xc[j3]+para[9]*Xc[j3+1];
		p2=exp(para[1]+para[2]*Xc[j3]+para[3]*Xc[j3+1]);
		p3=para[5];/*+para[6]*Xc[j3]+para[7]*Xc[j3+1];*/
		
		/*mexPrintf("%d,%d,%d\n",NObs,N1tmp,NLat);
		mexPrintf("%d,%d\n",NR,N0_ALL[i1]);*/
		
		for(j4=0;j4<N0_ALL[i1];j4++){
			/*n2 = j4*NXV;*/
			for(j1=obsloc[i1][j4];j1<obsloc[i1][j4+1];j1++){			
				j2=j1*NLat;
				n2 = j1*NXV;
				
				for(k1=0;k1<NLat;k1++){
					z1=z+i2+j2;				
					
					dtmp=dt[i4+j1];
					/*dtmps=sqrt(dtmp);*/
					
					dtmp1 = (z1[NLat+k1]-z1[k1]-drift1(z1,para,x+i1*NX,dtmp,xv+n2+n1)*dtmp);
					dtmp2 = diffu1(z1,para,x+i1*NX,dtmp,xv+n2+n1)*dtmp;
					sum1 += log(dtmp2) + dtmp1*dtmp1/dtmp2;
					
					
					/*mexPrintf("%f\n",sum1);*/
				}	
			}
		}
	}
	
	sum1*=-0.5;
	mexPrintf("DYN_sum1:%f\n",sum1);
	
	return sum1;
}


double dyn_likelihood_ME(double *Y, double *mu, double *Lamb, double *Omega, double *SIG, int NObs, int N01, int *N0_ALL, int NY, int NLat)
/*
z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
para- a vector of proposed regression coefficients
length of NX+1, including a intercept at location NX.
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1, i2, i3;
	int j1, j2, j3;
	int k1, k2;
	int n1, n2, n3;
	
	double sum1 = 0;
	double dtmp;
	
	double *logSIG;
	double *Ycur, *Ocur;
	logSIG = (double *)mxMalloc(sizeof(double)*NY);
	for(j1=0;j1<NY;j1++){
		logSIG[j1] = log(SIG[j1]);
	}
	
	for(i1=0;i1<NObs;i1++){
		i2 = i1 * N01 * NY;
		i3 = i1 * N01 * NLat;
		for(n1=1; n1<=N0_ALL[i1]; n1++){
			n2 = n1 * NY;
			n3 = n1 * NLat;
			
			Ycur = Y + i2 + n2;
			Ocur = Omega + i3 + n3;
			
			for(j1=0;j1<NY;j1++){
				dtmp = Ycur[j1] - mu[j1];
				for(k1=0;k1<NLat;k1++){
					dtmp -=Lamb[j1+k1*NY]*Ocur[k1];
				}
				sum1 += logSIG[j1] + dtmp * dtmp/SIG[j1];
			}
		}
	}
	
	sum1*=-0.5;
	
	mxFree(logSIG);
	
	return sum1;
}



int IMH(double *z, double *x, double *xv, double *dt, double *parac, double *param, double *paraCSig, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
parac- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1=0, i2;
	
	double *parap;
	double logpp, logpc;
	double loglp, loglc;
	double sum1;
	
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	
	rmt_gsl(param, paraCSig, parap, &logpp, Npara, df, r);	
	dmt(param, paraCSig, parac, &logpc, Npara, df);
	
	loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX, NXV, NLat);
	loglp = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX, NXV, NLat);	
	
	sum1 = exp(loglp + logpc - loglc - logpp);
	
	if(gsl_rng_uniform(r)<=sum1){
		for (i2 = 0; i2 < Npara; i2++) {					
			parac[i2] = parap[i2];
		}
		i1=1;
	}
	
	mxFree(parap);
	
	return i1;
}


int RWMH(double *z, double *x, double *xv, double *dt, double *parac, double *param, double *paraCSig, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
parac- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1=0, i2;
	
	double *parap;
	double logpp, logpc;
	double loglp, loglc;
	double sum1;
	double dtmp;
	
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	
	rmt_gsl(parac, paraCSig, parap, &logpp, Npara, df, r);	
	/*dmt(param, paraCSig, parac, &logpc, Npara, df);*/
	
	loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
	loglp = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
	
	/*mexPrintf("%f,%f\n",loglc,loglp);*/
	
	sum1 = exp(loglp - loglc);
	
	dtmp = gsl_rng_uniform(r);
	/*mexPrintf("%f\n",dtmp);*/
	if(dtmp<=sum1){
		for (i2 = 0; i2 < Npara; i2++) {					
			parac[i2] = parap[i2];
		}
		i1=1;
	}
	
	mxFree(parap);
	
	return i1;
}


double log_norm_den_mc(double x, double mean, double var){
	return(-0.5*(log(var)+(x-mean)*(x-mean)/var));
}

double log_gamma_den_mc(double x, double a, double b){
	return(-(a+1)*log(x)-b*x);
}


double log_prior_density(double *para, double *p1, double *p2, double *priortype, int Npara){
	int i1;
	double dtmp = 0;
	
	for(i1=0;i1<Npara;i1++){
		if(priortype[i1]==1){
			dtmp += (log_norm_den_mc(para[i1],p1[i1],p2[i1]));
		}else if(priortype[i1]==2){
			dtmp += (log_gamma_den_mc(para[i1],p1[i1],p2[i1]));
		}else{
			dtmp += 0;	
		}
	}
	
	return dtmp;
}

int IMH1(double *z, double *x, double *xv, double *dt, double *parac, double *param, double *paraCSig, double *p1, double *p2, double *priortype, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
parac- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1=0, i2;
	
	double *parap;
	double logpp, logpc;
	double loglp, loglc;
	double sum1;
	double logplp, logplc;
	
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	
	rmt_gsl(param, paraCSig, parap, &logplp, Npara, df, r);	
	dmt(param, paraCSig, parac, &logplc, Npara, df);
	
	loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
	loglp = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
	
	logpc = log_prior_density(parac,p1,p2,priortype,Npara);
	logpp = log_prior_density(parap,p1,p2,priortype,Npara);
	
	sum1 = exp(loglp + logpp + logplc - loglc -logpc - logplp);
	
	/*mexPrintf("%f,%f,%f,%f,%f,%f\n",loglp , logpp , logplc , loglc ,logpc , logplp);*/
	
	if(gsl_rng_uniform(r)<=sum1){
		for (i2 = 0; i2 < Npara; i2++) {					
			parac[i2] = parap[i2];
		}
		i1=1;
	}
	
	mxFree(parap);
	
	return i1;
}


int RWMH1(double *z, double *x, double *xv, double *dt, double *parac, double *param, double *paraCSig, double *p1, double *p2, double *priortype, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
parac- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1=0, i2;
	
	double *parap;
	double logpp, logpc;
	double loglp, loglc;
	double sum1;
	double dtmp;
	
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	
	
	rmt_gsl(parac, paraCSig, parap, &dtmp, Npara, df, r);	
	/*dmt(param, paraCSig, parac, &logpc, Npara, df);*/
	/*
	for(i2=0;i2<Npara;i2++){
	mexPrintf("%f ",parap[i2]);	
	}
	*/
	
	loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
	loglp = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
	
	logpc = log_prior_density(parac,p1,p2,priortype,Npara);
	logpp = log_prior_density(parap,p1,p2,priortype,Npara);
	
	/*
	mexPrintf("%f,%f\n",loglc,loglp);
	*/
	
	sum1 = exp(loglp +logpp - loglc - logpc);
	
	dtmp = gsl_rng_uniform(r);
	/*mexPrintf("%f\n",dtmp);*/
	if(dtmp<=sum1){
		for (i2 = 0; i2 < Npara; i2++) {					
			parac[i2] = parap[i2];
		}
		i1=1;
	}
	
	mxFree(parap);
	
	return i1;
}





int IMTM(double *z, double *x, double *xv, double *dt, double *para, double *param, double *paraCSig, double *p1, double *p2, double *priortype, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
para- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1, i2, iaccept=0;
	int NMTM = 5;
	
	double *parap, *parac;
	double logpp, logpc;
	double loglp, loglc;
	double logplp, logplc, logplc0, logplp0;
	double *logplpv;
	
	double *paracm, *parapm;
	double *omegap, *omegac;
	double *lomegap, *lomegac;
	
	int itmp1, itmp2, ichosen;
	double dtmp1, dtmp2;
	double dsum1, dsum2, dratio;
	
	/*holding values of parameters*/
	paracm = (double *) mxMalloc(sizeof(double) * Npara * NMTM);
	parapm = (double *) mxMalloc(sizeof(double) * Npara * NMTM);
	
	
	omegap = (double *) mxMalloc(sizeof(double) * NMTM);
	omegac = (double *) mxMalloc(sizeof(double) * NMTM);
	
	lomegap = (double *) mxMalloc(sizeof(double) * NMTM);
	lomegac = (double *) mxMalloc(sizeof(double) * NMTM);
	
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	parac=(double *) mxMalloc(sizeof(double) * Npara);
	
	logplpv = (double *) mxMalloc(sizeof(double) * NMTM);
	
	for(i1=0;i1<NMTM;i1++){
		rmt_gsl(param, paraCSig, parap, &logplp, Npara, df, r);			
		
		for(i2=0;i2<Npara;i2++){
			parapm[i1*Npara+i2] = parap[i2];
		}
		lomegap[i1] = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
		lomegap[i1] += log_prior_density(parap,p1,p2,priortype,Npara);
		
		logplpv[i1] = logplp;
		
		if(i1==0){
			dmt(param, paraCSig, para, &logplc, Npara, df);
			for(i2=0;i2<Npara;i2++){
				paracm[i1*Npara+i2] = para[i2];
			}
			lomegac[i1] = dyn_likelihood(z,x,xv,dt,para,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
			lomegac[i1] += log_prior_density(para,p1,p2,priortype,Npara);
			
			logplc0 = logplc;
		}else{
			rmt_gsl(param, paraCSig, parac, &logplc, Npara, df, r);
			for(i2=0;i2<Npara;i2++){
				paracm[i1*Npara+i2] = parac[i2];
			}
			lomegac[i1] = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);
			lomegac[i1] += log_prior_density(parac,p1,p2,priortype,Npara);
		}
		
		lomegap[i1] += logplc0;
	}
	
	itmp1 = which_max(lomegap,NMTM);
	dtmp1 = lomegap[itmp1];
	
	for(i1=0;i1<NMTM;i1++){
		omegap[i1] = exp(lomegap[i1] - dtmp1);		
	}
	
	ichosen = multinorm(omegap,NMTM,r);
	logplp0 = logplpv[ichosen];
	
	
	for(i1=0;i1<NMTM;i1++){
		lomegac[i1] += logplp0;
	}
	
	itmp2 = which_max(lomegac,NMTM);
	dtmp2 = lomegac[itmp2];
	
	for(i1=0;i1<NMTM;i1++){
		omegac[i1] = exp(lomegac[i1] - dtmp2);		
	}
	
	/*
	mexPrintf("%d,%d,%d\n",itmp1,itmp2,ichosen);
	mexPrintf("%f,%f\n",dtmp1,dtmp2);
	for(i1=0;i1<NMTM;i1++){
	mexPrintf("omega function:\n");
	mexPrintf("%f,%f\n",lomegap[i1],lomegac[i1]);
	}
	*/
	
	
	dsum1 = 0; dsum2 = 0;
	for(i1 = 0; i1<NMTM;i1++){
		dsum1 += omegap[i1];
		dsum2 += omegac[i1];		
	}
	
	dratio = exp(log(dsum1) - log(dsum2) + dtmp1 - dtmp2);
	
	/*
	mexPrintf("Ratio:%f\n",dratio);
	
	for(i2=0;i2<Npara;i2++){
	mexPrintf("%f ",parapm[ichosen*Npara + i2]);	
	}
	*/
	/*
	mexPrintf("%f\n",dratio);
	
	mexPrintf("%f,%f,%f,%f,%f,%f\n",loglp , logpp , logplc , loglc ,logpc , logplp);
	*/
	
	if(gsl_rng_uniform(r)<=dratio){
		for (i2 = 0; i2 < Npara; i2++) {					
			para[i2] = parapm[ichosen*Npara + i2];
		}
		iaccept=1;
	}
	
	mxFree(logplpv);
	
	mxFree(parac);
	mxFree(parap);
	
	mxFree(lomegac);
	mxFree(lomegap);
	
	mxFree(omegac);
	mxFree(omegap);
	
	mxFree(parapm);
	mxFree(paracm);
	
	
	return iaccept;
}

/*
void range_sol(double *drec_mt, double *betao_mt, int dim_bet, double tol, double range[])
{
double dre, ratio_bet;
int i1;

range[0]=-200000000;range[1]=200000000;
for(i1=0;i1<dim_bet-1;i1++){
dre=drec_mt[i1+1]-drec_mt[i1];
ratio_bet=(betao_mt[i1]-betao_mt[i1+1])/dre;
if(dre>0)range[0]=dmax2(range[0],ratio_bet);
else range[1]=dmin2(range[1],ratio_bet);
//Rprintf("%f,%f,%f\n",betao_mt[i1]-betao_mt[i1+1],dre,ratio_bet);
}
range[0]+=tol;range[1]-=tol;      
}
*/


int RWMH2(double *z, double *x, double *xv, double *dt, double *parac0, double *param, double *paraCSig, double *p1, double *p2, double *priortype, int df, int Npara, int NObs, int N1, int **obsloc, int *N0_ALL, int NX, int NXV, int NLat, gsl_rng *r, int *idx, int nidx, double mtis, double *constr_mat, int constr_mat_dim1, int *ineq_flag, double *a, int nmtm)
/*
Independent MH sampler

z- a vector of response variables in repeat measured, univariate
x- a vecto of covariates to form the subject specific regression coefficient exp(x^T\beta)
(x1',x2',...,xn')'
parac- a vector of current regression coefficients
length of NX+1, including a intercept at location NX.
param -- mean of the multivariate t distribution
paraCSig -- Cov of the multivariate t distribution
df -- df of the multivariate t distribution
Npara -- length of para
NObs- # of subjects
N0- maximum # of measurements accross all subjects.
NR- # of inserted points between two observed time points.
N0_ALL- a vector of # of measurement of all subjects
NX- length of x1'.
*/
{
	int i1, i2;
	int acc_flag=0;
	
	double *parap, *parac;
	double logpp, logpc;
	double loglp, loglc;
	double sum1;
	double dtmp,dtmp1;
	
	double *parap1, *parac1, *paraCSig1;
	
	double *drec_mt;
	
	double range_o[2], range_n[2], range_t[2];
	
	int nmtm1;
	
	double *rn_mt, *ro_mt, *parap1_mt, *parac1_mt;
	double *WKN_mt, *WKO_mt;
	double *mt_log_like, *prop_mt;
	
	double maxlog_n, maxlog_o;
	
	double temp1, temp2;
	
	parac=(double *) mxMalloc(sizeof(double) * Npara);
	parap=(double *) mxMalloc(sizeof(double) * Npara);
	parap1=(double *) mxMalloc(sizeof(double) * nidx);
	parac1=(double *) mxMalloc(sizeof(double) * nidx);
	paraCSig1 = (double *) mxMalloc(sizeof(double) * nidx * nidx);
	drec_mt = (double *) mxMalloc(sizeof(double) * nidx);
	
	rn_mt = (double *) mxMalloc(sizeof(double) * nmtm);
	ro_mt = (double *) mxMalloc(sizeof(double) * nmtm);
	parap1_mt=(double *) mxMalloc(sizeof(double) * nidx * nmtm);
	parac1_mt=(double *) mxMalloc(sizeof(double) * nidx * nmtm);
	
	WKN_mt = (double *) mxMalloc(sizeof(double) * nmtm);
	WKO_mt = (double *) mxMalloc(sizeof(double) * nmtm);
	mt_log_like = (double *) mxMalloc(sizeof(double) * nmtm);
	prop_mt = (double *) mxMalloc(sizeof(double) * nmtm);
	
	vb_eqn_va(parac0,parac,Npara);
	vb_eqn_va(parac,parap,Npara);
	
	submatrix1(paraCSig,paraCSig1,Npara,idx,nidx);
	subvector1(parac, parac1, idx, nidx);
	
	nmtm1 = nmtm -1;
	
	rmn_centered_gsl(paraCSig1, drec_mt, &dtmp, nidx, r);
	
	/*standardize direction*/
	dtmp=0.0;
	for(i1=0;i1<nidx;i1++)dtmp+=drec_mt[i1]*drec_mt[i1];
	dtmp=sqrt(dtmp);
	for(i1=0;i1<nidx;i1++)drec_mt[i1]/=dtmp;
	
	/*mexPrintf("Sol1\n");*/
	/*void range_sol2(double *X, double *beta, double *dvec, double *a, int *ineq_flag, int dimX, int *idx, int nidx, double tol, double range[])*/
	range_sol2(constr_mat, parac1, drec_mt, a, ineq_flag, constr_mat_dim1, idx, nidx, 0.00000001, range_o);
	/*     Rprintf("%f,%f\n",range_o[0],range_o[1]);*/
	
	for(i2=0;i2<nmtm;i2++){       
		rn_mt[i2]=trunom_gsl(mtis*range_o[0],mtis*range_o[1],r)/mtis;
		for(i1=0;i1<nidx;i1++){
			parap1[i1]=parap1_mt[i1+i2*nidx]=parac1[i1]+rn_mt[i2]*drec_mt[i1];
		}
		range_sol2(constr_mat, parap1, drec_mt, a, ineq_flag, constr_mat_dim1, idx, nidx, 0.00000001, range_t);	
		/*mexPrintf("Sol1_%d\n",i2);*/
		for(i1=0;i1<nidx;i1++){
			parap[idx[i1]] = parap1[i1];
			/*mexPrintf("%f ",parap1[i1]);*/
		}
		/*mexPrintf("\n");*/
		
		loglp = dyn_likelihood(z,x,xv,dt,parap,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
		logpp = log_prior_density(parap,p1,p2,priortype,Npara);
		
		mt_log_like[i2]=loglp + logpp; 
		prop_mt[i2]=-0.5*rn_mt[i2]*rn_mt[i2]*mtis*mtis-log(gsl_cdf_gaussian_P(range_t[1],1/mtis)-gsl_cdf_gaussian_P(range_t[0],1/mtis));  
	}
	maxlog_n=dmaxv(mt_log_like,nmtm);
	for(i2=0;i2<nmtm;i2++){
		WKN_mt[i2]=exp(mt_log_like[i2]-maxlog_n+prop_mt[i2]);
		/*            Rprintf("%f\n",WKN_mt[i2]);*/
	}
	/*i2=sample_int(WKN_mt,nmtm);*/
	i2 = multinorm(WKN_mt,nmtm,r);
	
	for(i1=0;i1<nidx;i1++){
		parap1[i1]=parap1_mt[i1+i2*nidx];
		parap[idx[i1]] = parap1[i1];
		parac1_mt[i1+(nmtm1)*nidx]=parac1[i1];
	}
	
	
	
	ro_mt[nmtm1]=rn_mt[i2];
	loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
	logpc = log_prior_density(parac,p1,p2,priortype,Npara);
	mt_log_like[nmtm1]=loglc + logpc;
	prop_mt[nmtm1]=-0.5*ro_mt[nmtm1]*ro_mt[nmtm1]*mtis*mtis-log(gsl_cdf_gaussian_P(range_o[1],1/mtis)-gsl_cdf_gaussian_P(range_o[0],1/mtis));
	
	/*mexPrintf("Sol2\n");*/
	range_sol2(constr_mat, parap1, drec_mt, a, ineq_flag, constr_mat_dim1, idx, nidx, 0.00000001, range_n);
	for(i2=0;i2<nmtm1;i2++){
		ro_mt[i2]=trunom_gsl(mtis*range_n[0],mtis*range_n[1],r)/mtis;;
		for(i1=0;i1<nidx;i1++){
			parac1_mt[i1+i2*nidx]=parac1[i1]=parap1[i1]+ro_mt[i2]*drec_mt[i1];
		}
		/*mexPrintf("Sol2_%d\n",i2);*/
		range_sol2(constr_mat, parac1, drec_mt, a, ineq_flag, constr_mat_dim1, idx, nidx, 0.00000001, range_t);	
		for(i1=0;i1<nidx;i1++){
			parac[idx[i1]] = parac1[i1];
			/*mexPrintf("%f ",parac1[i1]);*/
		}
		/*mexPrintf("\n");*/
		
		loglc = dyn_likelihood(z,x,xv,dt,parac,NObs,N1,obsloc,N0_ALL,NX,NXV,NLat);	
		logpc = log_prior_density(parac,p1,p2,priortype,Npara);
		
		mt_log_like[i2]=loglc + logpc;
		prop_mt[i2]=-0.5*ro_mt[i2]*ro_mt[i2]*mtis*mtis-log(gsl_cdf_gaussian_P(range_t[1],1/mtis)-gsl_cdf_gaussian_P(range_t[0],1/mtis));       
	}
	maxlog_o=dmaxv(mt_log_like,nmtm);
	for(i2=0;i2<nmtm;i2++)WKO_mt[i2]=exp(mt_log_like[i2]-maxlog_o+prop_mt[i2]);
	
	temp1=0.0;for(i1=0;i1<nmtm;i1++)temp1+=WKN_mt[i1];
	temp2=0.0;for(i1=0;i1<nmtm;i1++)temp2+=WKO_mt[i1];
	dtmp = gsl_rng_uniform(r);
	dtmp1=exp(maxlog_n+log(temp1)-(maxlog_o+log(temp2)));
	if(dtmp<dtmp1){
		for (i2 = 0; i2 < Npara; i2++) {					
			parac0[i2] = parap[i2];
		}
		acc_flag=1;
	}
	/*
	mexPrintf("parap:\n");
	printmatrix(parap,1,Npara);
	mexPrintf("parac:\n");
	printmatrix(parac,1,Npara);
	mexPrintf("p1:\n");
	printmatrix(p1,1,Npara);
	mexPrintf("p2:\n");
	printmatrix(p2,1,Npara);
	mexPrintf("priortype:\n");
	printmatrix(priortype,1,Npara);
	
	mexPrintf("%f,%f,%f,%f,%f,%f\n",loglp,logpp,loglc,logpc,dtmp,dtmp1);
	for(i1 = 0;i1<nidx;i1++){
		mexPrintf("%d ",idx[i1]);
	}
	mexPrintf("\n");
	*/
	
	if(acc_flag==1){
		mexPrintf("Accept\n");
	}else{
		mexPrintf("Reject\n");
	}
	
	/*rmt_gsl(parac1, paraCSig1, parap1, &dtmp, nidx, df, r);*/
	
	
	/*dmt(param, paraCSig, parac, &logpc, Npara, df);*/
	/*
	for(i2=0;i2<Npara;i2++){
	mexPrintf("%f ",parap[i2]);	
	}
	*/
	
	/*
	mexPrintf("%f,%f\n",loglc,loglp);
	*/
	
	mxFree(prop_mt);
	mxFree(mt_log_like);
	mxFree(WKO_mt);
	mxFree(WKN_mt);
	
	mxFree(parac1_mt);
	mxFree(parap1_mt);
	
	mxFree(rn_mt);
	mxFree(ro_mt);
	
	mxFree(drec_mt);
	mxFree(paraCSig1);
	mxFree(parac1);
	mxFree(parap1);
	mxFree(parap);
	mxFree(parac);
	
	return acc_flag;
}






void range_sol(double *drec_mt, double *betao_mt, int nidx, double tol, double range[])
{
	double dre, ratio_bet;
	int i1;
	
	range[0]=-200000000;range[1]=200000000;
	for(i1=0;i1<nidx-1;i1++){
		dre=drec_mt[i1+1]-drec_mt[i1];
		ratio_bet=(betao_mt[i1]-betao_mt[i1+1])/dre;
		if(dre>0)range[0]=dmax2(range[0],ratio_bet);
		else range[1]=dmin2(range[1],ratio_bet);
		/*        Rprintf("%f,%f,%f\n",betao_mt[i1]-betao_mt[i1+1],dre,ratio_bet);        */
	}
	range[0]+=tol;range[1]-=tol;      
}






void range_sol2(double *X, double *beta, double *dvec, double *a, int *ineq_flag, int dimX, int *idx, int nidx, double tol, double range[])
/*solve the range of lambda that satisfy the inequality X(beta+lambda*e)><0
X is a matrix, 
beta is a vector, 
lam is a scale,
e (dvec) is a directional vector
ineq_flag -- 1, >0, -1 <0

dimX -- number of row of X
nidx -- length of beta
tol -- very small positive value
range -- the range of lambda satisfy the inequalities
*/
{
	double lc1, lc2, ratio_bet;
	int tmpflag;
	int i1, i2;
	
	range[0]=-200000000;range[1]=200000000;
	for(i1=0;i1<dimX;i1++){
		if(ineq_flag[i1]!=0){
			lc1 =0; lc2 = 0;
			tmpflag = ineq_flag[i1];
			for(i2=0;i2<nidx;i2++){
				lc1 += X[i1+i2*dimX]*beta[i2];
				lc2 += X[i1+i2*dimX]*dvec[i2];
				/*mexPrintf("%f %f %f,",X[i1+i2*dimX],beta[i2],dvec[i2]);*/
			}
			/*mexPrintf("\n");*/
			
			if(lc2>0){
				tmpflag *= 1;
			}else if(lc2<0){
				tmpflag *= -1;
			}else{
				mexPrintf("range_sol2 error: denominator 0!\n");
				for(i2 = 0;i2<nidx;i2++){
					mexPrintf("%d ",idx[i2]);
				}
				mexPrintf("\n");
				
				mexPrintf("X:\n");
				for(i1=0;i1<dimX;i1++){
					for(i2=0;i2<nidx;i2++){
						mexPrintf("%f ",X[i1+i2*dimX]);
					}
					mexPrintf("\n");
				}
				mexPrintf("\n");
				
				mexPrintf("Current beta and direction vector:\n");
				for(i2=0;i2<nidx;i2++){
					mexPrintf("%f %f\n",beta[i2],dvec[i2]);
				}
				break;
			}
			
			ratio_bet=(a[i1]-lc1)/lc2;
			
			
			
			
			if(tmpflag==1)range[0]=dmax2(range[0],ratio_bet);
			else range[1]=dmin2(range[1],ratio_bet);
			/*        Rprintf("%f,%f,%f\n",beta[i1]-beta[i1+1],dre,ratio_bet);        */
		}
	}
	range[0]+=tol;range[1]-=tol;      
}




